from typing import Union
from functools import wraps

from fastapi import HTTPException, Request, status

from .models import User
from .database import db


def get_current_user(request: Request) -> User:
    if (user_sub := request.session.get("user_sub")) is None:
        raise HTTPException(status.HTTP_401_UNAUTHORIZED)
    return db.get_user(user_sub)


def get_current_user_or_none(request: Request) -> User | None:
    try:
        return get_current_user(request)
    except HTTPException:
        return None


def hasrole(required_roles: Union[str, list[str]] = []):
    required_roles_set: set[str]
    if isinstance(required_roles, str):
        required_roles_set = set([required_roles])
    else:
        required_roles_set = set(required_roles)

    def decorator(func):
        @wraps(func)
        async def wrapper(request=None, *args, **kwargs):
            if request is None:
                raise HTTPException(
                    500,
                    "Functions decorated with hasrole must have a request:Request argument",
                )
            user: User = get_current_user(request)
            if not any(required_roles_set.intersection(user.roles_as_set)):
                raise HTTPException(status.HTTP_401_UNAUTHORIZED)
            return await func(request, *args, **kwargs)

        return wrapper

    return decorator
